﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;

namespace EdgeClustering
{
    /// \author Manuel Keglevic
    /// \author Thomas Schulz
    /// <summary>
    /// This class is used to store points with additional functionality like sorting.
    /// </summary>
    public class ControlMeshVertex : IEquatable<ControlMeshVertex>, IComparable<ControlMeshVertex>
    {
        private double x;

        public double X
        {
            get { return x; }
            set { x = value; }
        }
        
        private double y;

        public double Y
        {
            get { return y; }
            set { y = value; }
        }

        public Point Point { get { return new Point(x, y); } }

        /// <summary>
        /// Creates a new Vertex
        /// </summary>
        /// <param name="x">x coordinate</param>
        /// <param name="y">y coordinate</param>
        public ControlMeshVertex(double x, double y)
        {
            this.x = x;
            this.y = y;
        }

        /// <summary>
        /// Creates a new Vertex
        /// </summary>
        /// <param name="p">point</param>
        public ControlMeshVertex(Point p)
        {
            this.x = p.X;
            this.y = p.Y;
        }

       
        public bool Equals(ControlMeshVertex other)
        {
             return Math.Abs(this.x - other.x) < 0.0001 && Math.Abs(other.y -this.y) < 0.0001;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null) return base.Equals(obj);

            if (!(obj is ControlMeshVertex))
                throw new InvalidCastException("The 'obj' argument is not a ControlMeshVertex object.");
            else
                return Equals(obj as ControlMeshVertex);
        }

        
        public int CompareTo(ControlMeshVertex other)
        {
            if (this.y < other.y)
                return -1;
            if (this.y > other.y)
                return 1;
            
            if (this.y == other.y)
            {
                if (this.x < other.x)
                    return -1;
                if (this.x > other.x)
                    return 1;
            }

            return 0;
        }
    }
}
