﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;

namespace EdgeClustering
{
    /// \author Manuel Keglevic
    /// \author Thomas Schulz
    /// <summary>
    /// Provides an Edge Class with advanced functions.
    /// </summary>
    class EnhancedEdge : IEquatable<EnhancedEdge>
    {
        private List<ControlMeshVertex> controlPoints = new List<ControlMeshVertex>();

        /// <summary>
        /// List of control points.
        /// </summary>
        public List<ControlMeshVertex> ControlPoints
        {
            get { return controlPoints; }
        }

        private SimpleEdge edge;

        public Point A
        {
            get { return edge.A; }
        }

        public Point B
        {
            get { return edge.B; }
        }

        /// <summary>
        /// Creates an EnhancedEdge using a SimpleEdge
        /// </summary>
        /// <param name="edge"></param>
        public EnhancedEdge(SimpleEdge edge)
        {
            this.edge = edge;
        }

        /// <summary>
        /// Sorts the control points depending on the distance to the startPoint.
        /// </summary>
        /// <param name="startPoint">start point for sorting</param>
        public void SortControlPoints(Point startPoint)
        {
            List<ControlMeshVertex> sortedPoints = new List<ControlMeshVertex>();

            double dist = 1000;
            int size = controlPoints.Count;
            int deleteIndex = 0;
            for (int i = 0; i < size; i++)
            {
                int count = controlPoints.Count;
                for (int d = 0; d < count; d++)
                {
                    double length = (controlPoints[d].Point - startPoint).Length;
                    if(length < dist){
                        dist = length;
                        deleteIndex = d;
                    }
                }

                sortedPoints.Add(controlPoints[deleteIndex]);
                controlPoints.RemoveAt(deleteIndex);
                dist = 1000;
            }

            controlPoints = sortedPoints;
        }
        /// <summary>
        /// Adds <code>p</code> to the list of control points, if it is not null.
        /// </summary>
        /// <param name="p">The new control point</param>
        public void AddControlPoint(Point p)
        {
            if (p != null)
            {
                ControlMeshVertex v = new ControlMeshVertex(p);
                if(!controlPoints.Contains(v))
                    controlPoints.Add(v);
            }
        }

        public bool Equals(EnhancedEdge other)
        {
            return this.edge.Equals(other.edge);
        }
    }
}
