﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;

namespace EdgeClustering
{
    /// \author Manuel Keglevic
    /// \author Thomas Schulz
    /// <summary>
    /// A simple class for storing edges which is used to represent the
    /// original graph from the xml file.
    /// </summary>
    public class SimpleEdge : IEquatable<SimpleEdge>
    {
        private Point a, b;
        private Vector v;
        private double alpha;

        /// <summary>
        /// Initial Point
        /// </summary>
        public Point A
        {
            get { return a; }
            set { a = value; update(); }
        }

        /// <summary>
        /// Terminal Point
        /// </summary>
        public Point B
        {
            get { return b; }
            set { b = value; update(); }
        }

        /// <summary>
        /// Directional vector representing the direction of the edge.
        /// </summary>
        public Vector V
        {
            get { return v; }
        }

        /// <summary>
        /// Angle of V in degree - counter clockwise between 0 (horizontal) , 90 (vertical) and 180 (horizontal)
        /// </summary>
        public double Alpha
        {
            get { return alpha; }
        }

        //Neuberechnung des Richtungsvektors und dessen Winkel
        private void update()
        {
            if (a != null && b != null)
            {
                v = b - a;
                v.Normalize();
                
                //Winkel des Richtungsvektors v 
                //Abfrage wegen Y notwendig, da Acos nur von 0-pi geht
                alpha = (v.Y >= 0) ? Math.Acos(v.X) : Math.Acos(-v.X);
                alpha *= 180.0 / Math.PI;
            }
        }
        public SimpleEdge(Point a,Point b)
        {
            this.a = a;
            this.b = b;
            update();
        }

        public bool Equals(SimpleEdge other)
        {
            return this.A.Equals(other.A) && this.B.Equals(other.B) || this.A.Equals(other.B) && this.B.Equals(other.A);           
        }
    }
}
