﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Xml;
using System.Collections;
using System.Windows.Media;
using System.Globalization;

namespace EdgeClustering
{
    /// \author Manuel Keglevic
    /// \author Thomas Schulz
    /// 
    /// <summary>
    /// An XML parser class to write and read data from or to XML files.
    /// </summary>
    static class XMLData
    {
        /// <summary>
        /// Writes the graph represented by <code>edges</code> to an XML file 
        /// with name <code>filename</code>.
        /// </summary>
        /// <param name="edges">A List of SimpleEdges which represent the 
        /// graph</param>
        /// <param name="filename">The name of the XML file where the graph 
        /// should be stored</param>
        public static void WriteLinkedNodes(List<SimpleEdge> edges, String filename)
        {
            XmlDocument doc = new XmlDocument();
            XmlNode root, node,rnode;

            root = doc.CreateElement("Edges");
            doc.AppendChild(root);

            foreach (SimpleEdge p in edges)
            {
                rnode = doc.CreateElement("Edge");
                root.AppendChild(rnode);

                node = doc.CreateElement("A");
                rnode.AppendChild(node);
                
                node.Attributes.Append(doc.CreateAttribute("x")).InnerText = p.A.X +"";
                node.Attributes.Append(doc.CreateAttribute("y")).InnerText = p.A.Y +"";

                node = doc.CreateElement("B");
                rnode.AppendChild(node);

                node.Attributes.Append(doc.CreateAttribute("x")).InnerText = p.B.X + "";
                node.Attributes.Append(doc.CreateAttribute("y")).InnerText = p.B.Y + "";
            }

            doc.Save(filename);
        }

        /// <summary>
        /// Reads graph data from the XML file with name <code>filename</code>.
        /// </summary>
        /// <param name="filename">The name of the graph input file</param>
        /// <returns>A List of SimpleEdges which represent the graph</returns>
        public static List<SimpleEdge> ReadLinkedNodes(String filename)
        {
            XmlDocument doc = new XmlDocument();
            //FIXME filename warad a hit
            doc.Load(filename);

            XmlElement root = doc.DocumentElement;
            XmlNodeList list = root.GetElementsByTagName("Edge");

            List<SimpleEdge> edges = new List<SimpleEdge>();

            foreach (XmlNode n in list)
            {
                XmlNode a, b;
                a = n["A"];
                b = n["B"];

                edges.Add(new SimpleEdge(
                            new Point(double.Parse(a.Attributes["x"].InnerText),
                                        double.Parse(a.Attributes["y"].InnerText)),
                            new Point(double.Parse(b.Attributes["x"].InnerText),
                                        double.Parse(b.Attributes["y"].InnerText))));
            }

            return edges;
        }

        /// <summary>
        /// Writes the current settings to an XML file with filename <code>filename</code>.
        /// </summary>
        /// <param name="filename">The name of the XML file where the settings 
        /// should be stored</param>
        public static void WriteSettings(String filename)
        {
            XmlDocument doc = new XmlDocument();
            XmlNode root, node, vals, visib, colors;

            root = doc.CreateElement("Settings");
            doc.AppendChild(root);

            // Graph settings
            vals = doc.CreateElement("Values");
            root.AppendChild(vals);

            node = doc.CreateElement("Border");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = Graph.Settings.Border + "";

            node = doc.CreateElement("GridSize");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = Graph.Settings.GridSize + "";

            node = doc.CreateElement("KdeHistogramSize");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = Graph.Settings.KdeHistogramSize + "";

            node = doc.CreateElement("KdeRadius");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = Graph.Settings.KdeRadius + "";

            node = doc.CreateElement("KdeThreshold");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = Graph.Settings.KdeThreshold + "";

            // GraphDrawer settings
            // bools
            visib = doc.CreateElement("Visibility");
            root.AppendChild(visib);

            node = doc.CreateElement("ControlMeshEdges");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.controlMeshEdgesVisible + "";

            node = doc.CreateElement("ControlMesh");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.controlMeshVisible + "";

            node = doc.CreateElement("ControlPoints");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.controlPointsVisible + "";

            node = doc.CreateElement("EnhancedGraph");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.enhancedGraphVisible + "";

            node = doc.CreateElement("GraphEdges");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.graphEdgesVisible + "";

            node = doc.CreateElement("KernelPeaks");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.kernelPeaksVisible + "";

            node = doc.CreateElement("MergedSegments");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.mergedSegmentsVisible + "";

            node = doc.CreateElement("Segments");
            visib.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.segmentsVisible + "";

            // colors
            colors = doc.CreateElement("Colors");
            root.AppendChild(colors);

            node = doc.CreateElement("ControlMesh");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.ControlMeshColor + "";

            node = doc.CreateElement("ControlMeshEdges");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.ControlMeshEdgesColor + "";

            node = doc.CreateElement("ControlPoints");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.ControlPointsColor + "";

            node = doc.CreateElement("EnhancedGraph");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.EnhancedGraphColor + "";

            node = doc.CreateElement("Graph");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.GraphColor + "";

            node = doc.CreateElement("KDEPeaks");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.KDEPeakColor + "";

            node = doc.CreateElement("MergedSegments");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.MergedSegmentsColor + "";

            node = doc.CreateElement("Segments");
            colors.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = GraphDrawer.Settings.SegmentsColor + "";

            // MergedSegment settings
            node = doc.CreateElement("MergedSegmentsThreshold");
            vals.AppendChild(node);
            node.Attributes.Append(doc.CreateAttribute("value")).InnerText = MergedSegment.threshold + "";

            doc.Save(filename);
        }

        /// <summary>
        /// Reads the settings from the XML file with filename <code>filename</code>.
        /// </summary>
        /// <param name="filename">The name of the settings file</param>
        public static void ReadSettings(String filename)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(filename);

            XmlElement root = doc.DocumentElement;
            XmlNodeList list = root.ChildNodes;

            foreach (XmlNode n in list)
            {
                XmlNode tmp;

                if (n.LocalName.Equals("Values"))
                {
                    tmp = n["Border"];
                    Graph.Settings.Border = double.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["GridSize"];
                    Graph.Settings.GridSize = int.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["KdeHistogramSize"];
                    Graph.Settings.KdeHistogramSize = int.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["KdeRadius"];
                    Graph.Settings.KdeRadius = double.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["KdeThreshold"];
                    Graph.Settings.KdeThreshold = double.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["MergedSegmentsThreshold"];
                    MergedSegment.threshold = double.Parse(tmp.Attributes["value"].InnerText);

                }
                else if (n.LocalName.Equals("Visibility"))
                {
                    tmp = n["ControlMeshEdges"];
                    GraphDrawer.Settings.controlMeshEdgesVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["ControlMesh"];
                    GraphDrawer.Settings.controlMeshVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["ControlPoints"];
                    GraphDrawer.Settings.controlPointsVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["EnhancedGraph"];
                    GraphDrawer.Settings.enhancedGraphVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["GraphEdges"];
                    GraphDrawer.Settings.graphEdgesVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["KernelPeaks"];
                    GraphDrawer.Settings.kernelPeaksVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["MergedSegments"];
                    GraphDrawer.Settings.mergedSegmentsVisible = bool.Parse(tmp.Attributes["value"].InnerText);

                    tmp = n["Segments"];
                    GraphDrawer.Settings.segmentsVisible = bool.Parse(tmp.Attributes["value"].InnerText);
                }
                else if (n.LocalName.Equals("Colors"))
                {
                    tmp = n["ControlMesh"];
                    GraphDrawer.Settings.ControlMeshColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["ControlMeshEdges"];
                    GraphDrawer.Settings.ControlMeshEdgesColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["ControlPoints"];
                    GraphDrawer.Settings.ControlPointsColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["EnhancedGraph"];
                    GraphDrawer.Settings.EnhancedGraphColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["Graph"];
                    GraphDrawer.Settings.GraphColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["KDEPeaks"];
                    GraphDrawer.Settings.KDEPeakColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["MergedSegments"];
                    GraphDrawer.Settings.MergedSegmentsColor = ParseColor(tmp.Attributes["value"].InnerText);

                    tmp = n["Segments"];
                    GraphDrawer.Settings.SegmentsColor = ParseColor(tmp.Attributes["value"].InnerText);
                }
            }
        }

        /// <summary>
        /// Parses a hexadecimal String of format <code>#AARRGGBB</code> and 
        /// returns an equivalent Color object.
        /// </summary>
        /// <param name="hex">The hex String which represents the color</param>
        /// <returns>A Color object, equivalent to the input String</returns>
        private static Color ParseColor(String hex)
        {
            byte a = byte.Parse(hex.Substring(1, 2), NumberStyles.HexNumber);
            byte r = byte.Parse(hex.Substring(3, 2), NumberStyles.HexNumber);
            byte g = byte.Parse(hex.Substring(5, 2), NumberStyles.HexNumber);
            byte b = byte.Parse(hex.Substring(7, 2), NumberStyles.HexNumber);
            return Color.FromArgb(a, r, g, b);
        }
    }
}
